%FDEMCOMP execute the inversion algorithm

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 1, 2020

figure(1), clf
figure(2), clf

if ~exist('app','var'), app.stop = 0; end

fdemprint( ' ', calledbyapp, textarea)
switch whattocomp
	case 1, fdemprint( 'Electrical conductivity inversion', calledbyapp, textarea)
	case 2, fdemprint( 'Magnetic permeability inversion', calledbyapp, textarea)
end

if syntheticdata
	fdemprint( 'Synthetic data', calledbyapp, textarea)
else
	fdemprint( 'Experimental data', calledbyapp, textarea)
end

if samenoise					% To use always the same noise
	rng(rngstatus)
end

switch modeltype
	case 1, fdemprint( 'Quadrature component of the signal (scaled)', calledbyapp, textarea)
	case 2, fdemprint( 'Quadrature component of the signal', calledbyapp, textarea)
	case 3, fdemprint( 'Inphase component of the signal', calledbyapp, textarea)
	case 4, fdemprint( 'Complex signal', calledbyapp, textarea)
end

switch whattocomp
	case 1
		switch modeltype		% define the model to invert
			case 1, func = @(sigma) aconduct(sigma,mu,h,d,R,omega,orientation,'sigma');
			case 2, func = @(sigma) quadracomp(sigma,mu,h,d,R,omega,orientation,'sigma');
			case 3, func = @(sigma) inphase(sigma,mu,h,d,R,omega,orientation,'sigma');
			case 4, func = @(sigma) hratio(sigma,mu,h,d,R,omega,orientation,'sigma');
		end
	case 2
		switch modeltype		% define the model to invert
			case 1, func = @(mu) aconduct(sigma,mu,h,d,R,omega,orientation,'mu');
			case 2, func = @(mu) quadracomp(sigma,mu,h,d,R,omega,orientation,'mu');
			case 3, func = @(mu) inphase(sigma,mu,h,d,R,omega,orientation,'mu');
			case 4, func = @(mu) hratio(sigma,mu,h,d,R,omega,orientation,'mu');
		end
end

%
% Inversion
%
if syntheticdata
	switch whattocomp
		case 1, truesol = funsigma(z);	% Vector of conductivity
		case 2, truesol = funmu(z);	% Vector of permeability
	end
	ytrue = func(truesol);			% Noiseless data
	% this scales the noise w.r.to the complex signal
	%funchr = @(sigma) hratio(sigma,mu,h,d,R,omega,orientation,'sigma');
	%funchr = @(mu) hratio(sigma,mu,h,d,R,omega,orientation,'mu');
	% this scales the noise w.r.to the data
	funchr = func;
	scal_ytrue = funchr(truesol);			% Noiseless signal
	y = addnoise(ytrue,ds,noisetype,scal_ytrue);	% Noisy data
	ymat = y;
else
	clear R freq h orientation m omega ds
	if ~fileavail
		if exist('./data','dir') == 7
			dirnam = 'data/*.mat';
		else
			dirnam = '*.mat';
		end
		[filnam,pathnam,filtind] = uigetfile(dirnam,'Open data file');
		fullpathname = [pathnam filnam];
	end
	load(fullpathname)
	str = sprintf('Field data: %s', FDEMdata.name);
	fdemprint(str, calledbyapp, textarea);
	R = FDEMdata.R;
	freq = FDEMdata.freq;
	h = FDEMdata.height(:);
	if forceorient==1
		FDEMdata.orient='vertical';
	elseif forceorient==2
		FDEMdata.orient='horizontal';
	elseif forceorient==3
		FDEMdata.orient='perp';
	end
	orientation = FDEMdata.orient;

	switch FDEMdata.orient(1:2)
		case 've', ymat = FDEMdata.vdata;
		case 'ho', ymat = FDEMdata.hdata;
		case 'pe', ymat = [FDEMdata.pdata];
		case 'vh', ymat = [FDEMdata.vdata;FDEMdata.hdata];
		case 'vp', ymat = [FDEMdata.vdata;FDEMdata.pdata];
		otherwise, error('Error in data format.')
	end
	if FDEMdata.type(1)=='c'
		switch modeltype
			case 2, ymat = imag(ymat);
			case 3, ymat = real(ymat);
			case 4, % ymat = ymat;
			otherwise, error('Bad model.')
		end
	elseif ((FDEMdata.type(1)=='i') && (modeltype>2)) || ...
			((FDEMdata.type(1)=='r') && (modeltype~=3))
		error('Bad model.')
	end
	if averagecols, ymat = mean(ymat,2); end
	m = size(h,1);			% number of heights
	nR = length(R);			% number of IC spacings
	nf = length(freq);		% number of frequencies
	omega = 2*pi*freq;
%	if removenegativedata
%		% only for data with negative values (explorer/mini-explorer)
%		ineg = any(ymat'<=0);
%		str = sprintf('Negative data at rows: %d %d\n',find(ineg));
%		fdemprint(str,calledbyapp, textarea)
%		ymat(ineg,:) = nan;
%		if size(ymat,2)==1
%			yrows = find(~isnan(ymat));
%		else
%			yrows = find(~any(isnan(ymat')));
%		end
%		ymat = ymat(yrows,:);
%	end
	y = ymat(:,coltoinvert);	% select one column of dataset
	ytrue = y;
	truesol = [];
	ds = 0;
end

ndata = size(y,1);
if ~isreal(y), ndata = 2*ndata; end
str = sprintf('orientation: %s\nfreq=', orientation);
str = [str sprintf('%d ', freq)];
str = [str sprintf('\nR=')];
str = [str sprintf('%g ', R)];
str = [str sprintf('\nnh=%d, nR=%d, nf=%d, n.data=%d, ', m, nR, nf, ndata)];
str = [str sprintf('n=%d, noise=%.2e\n', n, ds)];
str = [str sprintf('Regularizing term: %d-derivative\n',Lind)];
str = [str sprintf('rel. residual is normalized by the norm of the data vector\n')];
fdemprint( str, calledbyapp, textarea)

if Lind==0		% set number of kappa values
	p = min(ndata,n);
else
	p = size(get_l(n,Lind),1) - max(n-ndata,0);
end

if p<=0
	if calledbyapp
		uialert(fdemui, 'Too few data to invert. You must increase the number of data values.', 'Warning', 'Icon', 'Error')
	end
	error('Too few data to invert. You must increase the number of data values.')
end

opts = struct( 'Lind', Lind, ...
	'Ltype', Ltype, ...
	'sigmam', sigmam, ...
	'epsilon', epsilon, ...
	'orientation', orientation, ...
	'metjac', metjac, ...
	'tau', tau, ...
	'nmax', nmax, ...
	'damped', damped, ...
	'dampos', dampos, ...
	'beta', beta, ...
	'mnflag', mnflag, ...
	'show', show, ...
	'rowcol', [2 4], ...
	'metk', {metk}, ...
	'taudiscr', taudiscr, ...
	'truesol', truesol, ...
	'ds', ds, ...
	'r_in', r_in);

if ~invertall, ymat = y; end
ncol = size(ymat,2);
if ncol>1
	str = sprintf('Number of columns to process: %d',ncol);
	fdemprint( str, calledbyapp, textarea)
	if showpcolor
		figure(3), clf
		set(gcf,'NumberTitle','off','name','2D reconstruction');
		axis ij, axis tight
		axis([1 ncol z(1) z(end)])
		h1 = colorbar;
		h2 = get(h1,'label');
		switch whattocomp
			case 1, set(h2,'string','Electrical conductivity (S/m)');
			case 2, set(h2,'string','Magnetic Permeability (H/m)');
		end
		set(h2,'fontsize',11)
		xlabel('Number of soundings','fontsize',11)
		ylabel('Depth (m)','fontsize',11)
		hold on
	end
end

% load initial model to file
if modelavail
	clear n lastz z d sigmainit muinit
	if ~filmodavail
		if exist('./data','dir') == 7
			dirnam = 'data/*.mat';
		else
			dirnam = '*.mat';
		end
		[filnam1,pathnam1,filtind1] = uigetfile(dirnam,'Open input model file');
		fullpathname1 = [pathnam1 filnam1];
	end
	load(fullpathname1)
	colsigma = size(FDEMinputmodel.sigmamodel,2);
	if (colsigma~=1) && (colsigma~=ncol)
		if calledbyapp
			uialert(app.FDEM, 'Bad initial model. Column number of data and input model must agree!', 'Warning', 'Icon', 'Error')
		end
		error ('Bad initial model. Column number of data and input model must agree!')
	end
	colmu = size(FDEMinputmodel.mumodel,2);
	if (colmu~=1) && (colmu~=ncol)
		if calledbyapp
		uialert(app.FDEM, 'Bad initial model. Dimensions between data and input model must be agreed!', 'Warning', 'Icon', 'Error')
		error ('Bad initial model. Dimensions between data and input model must be agreed!')
		return
		else
		error ('Bad initial model. Dimensions between data and input model must be agreed!')
		end
	end
	str = sprintf('Input model: %s', FDEMinputmodel.name);
	fdemprint(str, calledbyapp, textarea);
	z = FDEMinputmodel.z;
	n = size(z,1);
	lastz = z(end);
	d = (lastz/(n-1))*ones(n-1,1);
	if any(FDEMinputmodel.mumodel > 1e-3)
		warning('Large permeability values! The permeability should be absolute, not relative.')
	end
	switch whattocomp
		case 1
			sigmainit = FDEMinputmodel.sigmamodel;
			mu = FDEMinputmodel.mumodel;
		case 2
			muinit = FDEMinputmodel.mumodel;
			sigma = FDEMinputmodel.sigmamodel;
	end
end

% turn initial model to a matrix, if it isn't already
switch whattocomp
	case 1
		if size(sigmainit,2) < ncol
			sigmainit = repmat(sigmainit,1,ncol);
		end
		if size(mu,2) < ncol
			mu = repmat(mu,1,ncol);
		end
	case 2
		if size(muinit,2) < ncol
			muinit = repmat(muinit,1,ncol);
		end
		if size(sigma,2) < ncol
			sigma = repmat(sigma,1,ncol);
		end
end

SOL = zeros(n,ncol);
kappavet = zeros(ncol,1);
for col = 1:ncol
	if ncol>1
		str = sprintf('Processing data column %d',col);
		fdemprint( str, calledbyapp, textarea)
	end
	y = ymat(:,col);
	switch whattocomp
		case 1
			switch modeltype		% define the model to invert
				case 1, func = @(sigma) aconduct(sigma,mu(:,col),h,d,R,omega,orientation,'sigma');
				case 2, func = @(sigma) quadracomp(sigma,mu(:,col),h,d,R,omega,orientation,'sigma');
				case 3, func = @(sigma) inphase(sigma,mu(:,col),h,d,R,omega,orientation,'sigma');
				case 4, func = @(sigma) hratio(sigma,mu(:,col),h,d,R,omega,orientation,'sigma');
			end
			opts.sigmainit = sigmainit(:,col);
			opts.mu = mu(:,col);
			if usexbar
				opts.xbar = opts.sigmainit;
			else
				opts.xbar = zeros(size(opts.sigmainit));
			end
			[sigma,kappa,rho,eta,X,err,fail] = emsolvenlsig(func,y,h,d,opts,textarea,app);
			SOL(:,col) = sigma(:,1);
			kappavet(col) = kappa(1);
		case 2
			switch modeltype		% define the model to invert
				case 1, func = @(mu) aconduct(sigma(:,col),mu,h,d,R,omega,orientation,'mu');
				case 2, func = @(mu) quadracomp(sigma(:,col),mu,h,d,R,omega,orientation,'mu');
				case 3, func = @(mu) inphase(sigma(:,col),mu,h,d,R,omega,orientation,'mu');
				case 4, func = @(mu) hratio(sigma(:,col),mu,h,d,R,omega,orientation,'mu');
			end
			opts.muinit = muinit(:,col);
			opts.sigma = sigma(:,col);
			if usexbar
				opts.xbar = opts.muinit;
			else
				opts.xbar = zeros(size(opts.muinit));
			end
			[mu,kappa,rho,eta,X,err,fail] = emsolvenlmu(func,y,h,d,opts,textarea,app);
			SOL(:,col) = mu(:,1);
			kappavet(col) = kappa(1);
	end
	if showpcolor && (col>1)
		figure(3)
		pcolor([col-1:col]',z,SOL(:,col-1:col))
		shading interp
	end
	if app.stop, break, end
end
if showpcolor && (ncol>1), figure(3), hold off, end


